var Coleccion = function(l, options) {
    this.lista = [];
    this.cargando = false;
    this.seleccion = null;
    this.validarSeleccion = true;
    this.limpiarAlCargar = true;

    if (typeof Object.observe === 'function') {
        var coleccion = this;
        Object.observe(coleccion, function(changes){

            changes.forEach(function(change) {
                switch (change.name) {
                    case 'lista': {
                        if (!coleccion.lista || (coleccion.lista && (coleccion.lista instanceof Array) && coleccion.lista.indexOf(coleccion.seleccion) < 0)) {
                            coleccion.seleccion = null;
                        }

                        return false;
                    } break;

                    case 'seleccion': {
                        if (coleccion.validarSeleccion && coleccion.seleccion && coleccion.seleccion != change.oldValue) {
                            if (coleccion.lista && (coleccion.lista instanceof Array) && coleccion.lista.indexOf(coleccion.seleccion) < 0) {
                                coleccion.seleccion = null;
                            }
                        }

                        return false;
                    } break;

                    default: return true;
                }
            });

        });
    }

    if (l && (l instanceof Array)) {
        this.lista = l;
    } else if (l && options === undefined) {
        options = l;
    }

    if (options === undefined) {
        options = { };
    }

    this.validarSeleccion = (options.validarSeleccion !== undefined && options.validarSeleccion != null) ? options.validarSeleccion : true;
    this.limpiarAlCargar = (options.limpiarAlCargar !== undefined && options.limpiarAlCargar != null) ? options.limpiarAlCargar : true;

    if (options.seleccionarPrimero && this.lista.length > 0) {
        this.seleccionar(this.lista[0]);
    }
};

Coleccion.prototype.cargar = function(f) {
    if (!f || (typeof f !== 'function')) return;

    var coleccion = this;

    if (coleccion.limpiarAlCargar)
        coleccion.lista = [];

    coleccion.cargando = true;

    var finalizar = function() {
        coleccion.cargando = false;
    };

    try {
        var promise = f();
        if (promise && promise.finally && (typeof promise.finally === 'function')) {
            promise.finally(finalizar);
        } else {
            finalizar();
        }
    } catch (err) {
        finalizar();
    }
};

Coleccion.prototype.seleccionar = function(filter) {
    if (!filter || !this.lista || this.lista.length == 0) {
        this.seleccion = null;
        return false;
    }

    var seleccionado = false;

    if (typeof filter === 'function') {
        var grep = $.grep(this.lista, filter);
        if (grep.length > 0) {
            this.seleccion = grep[0];
            seleccionado = true;
        }
    } else {
        var idx = this.lista.indexOf(filter);
        if (idx >= 0) {
            this.seleccion = this.lista[idx];
            seleccionado = true;
        }
    }

    return seleccionado;
};
